@@ -9,19 +9,23 @@ import datetime
 import time
 import os
 import glob
+import matplotlib.font_manager as fm
 
 
 class ConfuseCallback(tf.keras.callbacks.Callback):
+    """
+    Generate Confusion Matrix and write an image to TensorBoard
+    """
     def __init__(self, x_test, y_test, file_writer, dataset=None, class_names=None, figure_size=(12, 10), batch_size=32):
         """
-
-        :param x_test: (array) - (n data, data dimension(Ex. 32x32x3 or 600x30 ..., etc). If None is given, dataset must be provided.
-        :param y_test: (array) - (n data, ). If None is given, dataset must be provided.
-        :param file_writer: TensorBoard File Writer
-        :param dataset: (TensorFlow Dataset). If dataset is given, x_test and y_test is ignored. Default: None.
-        :param class_names: (array(str)) - Names of class. If None, default names are set to (Class01, Class02 ...). Default: None.
-        :param figure_size: (tuple) - Figure size of confusion matrix. Default: (12, 10).
-        :param batch_size: (int) - Batch size to predict x_test. If dataset is given, batch_size is ignored and batch size set in dataset is used.
+        Args:
+            x_test (np.ndarray): (n data, data dimension(Ex. 32x32x3 or 600x30 ..., etc). If None is given, dataset must be provided.
+            y_test (np.ndarray): (n data, ). If None is given, dataset must be provided.
+            file_writer (tf.summary.SummaryWriter): TensorBoard File Writer
+            dataset (tf.keras.dataset.Dataset): If dataset is given, x_test and y_test is ignored. Default: None.
+            class_names (list of str): Names of class. If None, default names are set to (Class01, Class02 ...). Default: None.
+            figure_size (tuple): Figure size of confusion matrix. Default: (12, 10).
+            batch_size (int): Batch size to predict x_test. If dataset is given, batch_size is ignored and batch size set in dataset is used.
         """
         super(ConfuseCallback, self).__init__()
         self.dataset = dataset
@@ -45,7 +49,24 @@ class ConfuseCallback(tf.keras.callbacks.Callback):
         if self.label_names is None and self.y_test is not None:
             self.label_names = ["Class {:02d}".format(unique_label) for unique_label in np.unique(self.y_test)]
 
+        # For Korean Label
+        font_list = [font.name for font in fm.fontManager.ttflist]
+        possible_fonts = [font_name for font_name in font_list if font_name.find("CJK") >=0]
+        if len(possible_fonts) > 0:
+            plt.rcParams['axes.unicode_minus'] = False
+            plt.rcParams['font.family'] = possible_fonts[0]
+
     def get_precision_recall_plot(self, con_mat):
+        """
+        Generate Precision and Recall plot bar plot image
+        Args:
+            con_mat (np.ndarray): Confusion Matrix array
+
+        Returns:
+            tf.TensorArray: Precision and Recall Bar Plot Image
+            np.ndarray: Precisions
+            np.ndarray: Recalls
+        """
         precisions = np.array([0] * len(self.label_names)).astype('float32')
         recalls = np.array([0] * len(self.label_names)).astype('float32')
 
@@ -134,14 +155,18 @@ class ConfuseCallback(tf.keras.callbacks.Callback):
 
 
 class ModelSaverCallback(tf.keras.callbacks.Callback):
+    """
+    Saves Model at each end of the epoch when the best accuracy/loss is presented.
+    """
     def __init__(self, best_metric=float('inf'), save_root="./", save_metric='val_loss', enable=True, epoch=0):
         """
 
-        :param best_metric: Set best score of previous training session if resuming
-        :param save_root: Model save path
-        :param save_metric: One of 'val_loss', 'val_accuracy'
-        :param enable: (bool)
-        :param epoch: Set previous epoch number if resuming
+        Args:
+            best_metric (float): Set best score of previous training session if resuming.
+            save_root (str): Model save path
+            save_metric (str): One of 'val_loss', 'val_accuracy'
+            enable (bool): Set previous epoch number if resuming
+            epoch (int): Epoch number
         """
         super(ModelSaverCallback, self).__init__()
         self.best_metric = best_metric
@@ -187,7 +212,19 @@ class ModelSaverCallback(tf.keras.callbacks.Callback):
 
 
 class SparsityCallback(tf.keras.callbacks.Callback):
+    """
+    Computes the sparsity on each layer of the given model and saves bar plot image to the TensorBoard.
+    """
     def __init__(self, file_writer, sparsity_threshold=0.05, figure_size=(12, 20)):
+        """
+
+        Args:
+            file_writer (tf.summary.SummaryWriter): TensorBoard File Writer
+            sparsity_threshold (float): Sparsity Threshold of each layer.
+                                        Ex) 0.05 -> Find the number of weights where -0.05 < values < 0.05 in a layer.
+                                        Percentage of the number if set to the sparsity of the layer.
+            figure_size (tuple): Figure size to generate plot image
+        """
         super(SparsityCallback, self).__init__()
 
         self.file_writer = file_writer
@@ -195,6 +232,16 @@ class SparsityCallback(tf.keras.callbacks.Callback):
         self.figure_size = list(figure_size)
 
     def get_sparsity_plot(self, sparse_levels, sparse_layer_names):
+        """
+        Generate sparsity plot image
+
+        Args:
+            sparse_levels (np.ndarray): Sparse levels for the layer
+            sparse_layer_names (list of str, np.ndarray): Names of layer along with sparse_levels list
+
+        Returns:
+
+        """
         width = 0.8
 
         n_data = sparse_levels.shape[0]
@@ -257,6 +304,18 @@ class SparsityCallback(tf.keras.callbacks.Callback):
 
 
 def run_tensorboard(path, host='0.0.0.0', port=6006):
+    """
+    Run TensorBoard in python script.
+    Args:
+        path (str): TensorBoard log dir
+        host(str): Host address for TensorBoard.
+                    127.0.0.1 -> localhost.
+                    0.0.0.0 -> Allow remote connection.
+        port (int): Port number for TensorBoard
+
+    Returns:
+        None
+    """
     tb = program.TensorBoard()
     tb.configure(argv=[None, '--logdir', path, '--host', host, '--port', f"{port:}"])
     url = tb.launch()
@@ -267,6 +326,15 @@ def run_tensorboard(path, host='0.0.0.0', port=6006):
 
 
 def wait_ctrl_c(pre_msg="Press Ctrl+c to quit Tensorboard", post_msg="\nExit."):
+    """
+    Wait until ctrl+c is pressed. This function is to prevent quitting python process when the training is completed when TensorBoard is running.
+    Args:
+        pre_msg: Message prior to wait ctrl+c
+        post_msg: Message post to ctrl+c pressed
+
+    Returns:
+        None
+    """
     print(pre_msg)
     try:
         while True:
@@ -282,34 +350,43 @@ def get_tf_callbacks(root,
                      earlystop_callback=True, earlystop_monitor='val_loss', earlystop_patience=0, earlystop_restore_weights=True,
                      sparsity_callback=False, sparsity_threshold=0.05):
     """
-    Get TensorFlow callbacks function for convinience
-    :param root: Root directory for TensorBoard
-    :param tboard_callback: (bool) - Whether using TensorBoard or not. Default: True
-    :param tboard_update_freq: (str) - TensorBoard update frequency. ('epoch', 'batch'). Default: 'epoch'
-    :param tboard_histogram_freq: (int) - TensorBoard histogram update frequency. Default: 1
-    :param tboard_profile_batch: (int) - TensorBoard profile timing. If 0 is given, profiling is not used.
-                                        Ex) If 10 is given, profiling is executed at batch of 10. Default: 0
-    :param confuse_callback: (bool) - Whether using confusion matrix for TensorBoard callback or not.
-                                      At least one of the following three ((x_test, y_test), test_generator_, test_dataset) must be set.
-                                      Otherwise, Confusion Matrix callback will be ignored.
-                                      Default: True.
-    :param label_info: (None or array(str))-  Default: None
-    :param x_test: (None or array) - Default: None
-    :param y_test: (None or array) = Default: None
-    :param test_generator_: (None or HDF5Generator) - Default: None. For HDF5Generator test set purpose.
-    :param test_dataset: (None or TensorFlow Dataset) - Default: None.
-    :param figure_size: (tuple) - Figure Size of Confusion Matrix.
-    :param modelsaver_callback: (bool) - Whether using ModelSaver callback or not. Saving the model file when the lowest validation loss is given per each epochs.
+    Getting TensorFlow callbacks function for convenience purpose.
+    Args:
+        root (str): Root directory for TensorBoard
+        tboard_callback (bool): Whether using TensorBoard or not. Default: True
+        tboard_update_freq (str): TensorBoard update frequency. ('epoch', 'batch'). Default: 'epoch'
+        tboard_histogram_freq (int): TensorBoard histogram update frequency. Default: 1
+        tboard_profile_batch (int): TensorBoard profile timing. If 0 is given, profiling is not used.
+                                Ex) If 10 is given, profiling is executed at batch of 10. Default: 0
+        confuse_callback (bool): Whether using confusion matrix for TensorBoard callback or not.
+                          At least one of the following three ((x_test, y_test), test_generator_, test_dataset) must be set.
+                          Otherwise, Confusion Matrix callback will be ignored.
+                          Default: True.
+        label_info (list of str): Names of class. If None, default names are set to (Class01, Class02 ...). Default: None.
+        x_test (np.ndarray, None): (n data, data dimension(Ex. 32x32x3 or 600x30 ..., etc). If None is given, dataset must be provided.
+        y_test (np.ndarray, None): (n data, ). If None is given, dataset must be provided.
+        test_generator_ (tfhelper.dataset.HDF5Generator, None): Default: None. For HDF5Generator test set purpose.
+        test_dataset (tf.dataset.Dataset, None): Default: None.
+        figure_size (tuple): Figure Size of Confusion Matrix.
+        modelsaver_callback (bool): Whether using ModelSaver callback or not. Saving the model file when the lowest validation loss is given per each epochs.
                                         Default: False.
-    :param best_loss:
-    :param save_root:
-    :param best_epoch:
-    :param save_metric:
-    :param earlystop_callback:
-    :param earlystop_monitor:
-    :param earlystop_patience:
-    :param earlystop_restore_weights:
-    :return:
+        best_loss (float): Set best score of previous training session if resuming.
+        save_root (str): Model save path
+        best_epoch (int): Previous Best epoch number if resuming
+        save_metric (str): One of 'val_loss', 'val_accuracy'
+        earlystop_callbac (bool): Early Stop callback
+        earlystop_monitor (str): Earlys top_monitor metric 'val_loss', 'val_accuracy'
+        earlystop_patience (int): Early stop patience
+        earlystop_restore_weights (bool): Restore weights on early stop.
+        sparsity_callback (bool): Sparsity callback.
+        sparsity_threshold (float): Sparsity Threshold of each layer.
+                            Ex) 0.05 -> Find the number of weights where -0.05 < values < 0.05 in a layer.
+                            Percentage of the number if set to the sparsity of the layer.
+
+    Returns:
+        list of tf.keras.callbacks.Callback: Callback List
+        str: Tensor Board Log Root Directory
+
     """
     postfix = datetime.datetime.now().strftime("%Y%m%d-%H%M%S")
     log_root_ = "{}{}/".format(root, postfix)
